from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.message_priority import MessagePriority;
from ..enums.smtp_encryption import SmtpEncryption;

class Messaging(Service):

    def __init__(self, client) -> None:
        super(Messaging, self).__init__(client)

    def list_messages(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all messages from the current Rilz project.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: scheduledAt, deliveredAt, deliveredTotal, status, description, providerType
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_email(self, message_id: str, subject: str, content: str, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, cc: List[str] = None, bcc: List[str] = None, attachments: List[str] = None, draft: bool = None, html: bool = None, scheduled_at: str = None) -> Dict[str, Any]:
        """
        Create a new email message.

        Parameters
        ----------
        message_id : str
            Message ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        subject : str
            Email Subject.
        content : str
            Email Content.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        cc : List[str]
            Array of target IDs to be added as CC.
        bcc : List[str]
            Array of target IDs to be added as BCC.
        attachments : List[str]
            Array of compound ID strings of bucket IDs and file IDs to be attached to the email. They should be formatted as <BUCKET_ID>:<FILE_ID>.
        draft : bool
            Is message a draft
        html : bool
            Is content of type HTML
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/email'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        if subject is None:
            raise RilzException('Missing required parameter: "subject"')

        if content is None:
            raise RilzException('Missing required parameter: "content"')


        api_params['messageId'] = message_id
        api_params['subject'] = subject
        api_params['content'] = content
        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['cc'] = cc
        api_params['bcc'] = bcc
        api_params['attachments'] = attachments
        api_params['draft'] = draft
        api_params['html'] = html
        api_params['scheduledAt'] = scheduled_at

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_email(self, message_id: str, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, subject: str = None, content: str = None, draft: bool = None, html: bool = None, cc: List[str] = None, bcc: List[str] = None, scheduled_at: str = None, attachments: List[str] = None) -> Dict[str, Any]:
        """
        Update an email message by its unique ID. This endpoint only works on messages that are in draft status. Messages that are already processing, sent, or failed cannot be updated.
        

        Parameters
        ----------
        message_id : str
            Message ID.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        subject : str
            Email Subject.
        content : str
            Email Content.
        draft : bool
            Is message a draft
        html : bool
            Is content of type HTML
        cc : List[str]
            Array of target IDs to be added as CC.
        bcc : List[str]
            Array of target IDs to be added as BCC.
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        attachments : List[str]
            Array of compound ID strings of bucket IDs and file IDs to be attached to the email. They should be formatted as <BUCKET_ID>:<FILE_ID>.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/email/{messageId}'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)

        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['subject'] = subject
        api_params['content'] = content
        api_params['draft'] = draft
        api_params['html'] = html
        api_params['cc'] = cc
        api_params['bcc'] = bcc
        api_params['scheduledAt'] = scheduled_at
        api_params['attachments'] = attachments

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_push(self, message_id: str, title: str = None, body: str = None, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, data: dict = None, action: str = None, image: str = None, icon: str = None, sound: str = None, color: str = None, tag: str = None, badge: float = None, draft: bool = None, scheduled_at: str = None, content_available: bool = None, critical: bool = None, priority: MessagePriority = None) -> Dict[str, Any]:
        """
        Create a new push notification.

        Parameters
        ----------
        message_id : str
            Message ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        title : str
            Title for push notification.
        body : str
            Body for push notification.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        data : dict
            Additional key-value pair data for push notification.
        action : str
            Action for push notification.
        image : str
            Image for push notification. Must be a compound bucket ID to file ID of a jpeg, png, or bmp image in Rilz Storage. It should be formatted as <BUCKET_ID>:<FILE_ID>.
        icon : str
            Icon for push notification. Available only for Android and Web Platform.
        sound : str
            Sound for push notification. Available only for Android and iOS Platform.
        color : str
            Color for push notification. Available only for Android Platform.
        tag : str
            Tag for push notification. Available only for Android Platform.
        badge : float
            Badge for push notification. Available only for iOS Platform.
        draft : bool
            Is message a draft
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        content_available : bool
            If set to true, the notification will be delivered in the background. Available only for iOS Platform.
        critical : bool
            If set to true, the notification will be marked as critical. This requires the app to have the critical notification entitlement. Available only for iOS Platform.
        priority : MessagePriority
            Set the notification priority. "normal" will consider device state and may not deliver notifications immediately. "high" will always attempt to immediately deliver the notification.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/push'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')


        api_params['messageId'] = message_id
        api_params['title'] = title
        api_params['body'] = body
        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['data'] = data
        api_params['action'] = action
        api_params['image'] = image
        api_params['icon'] = icon
        api_params['sound'] = sound
        api_params['color'] = color
        api_params['tag'] = tag
        api_params['badge'] = badge
        api_params['draft'] = draft
        api_params['scheduledAt'] = scheduled_at
        api_params['contentAvailable'] = content_available
        api_params['critical'] = critical
        api_params['priority'] = priority

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_push(self, message_id: str, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, title: str = None, body: str = None, data: dict = None, action: str = None, image: str = None, icon: str = None, sound: str = None, color: str = None, tag: str = None, badge: float = None, draft: bool = None, scheduled_at: str = None, content_available: bool = None, critical: bool = None, priority: MessagePriority = None) -> Dict[str, Any]:
        """
        Update a push notification by its unique ID. This endpoint only works on messages that are in draft status. Messages that are already processing, sent, or failed cannot be updated.
        

        Parameters
        ----------
        message_id : str
            Message ID.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        title : str
            Title for push notification.
        body : str
            Body for push notification.
        data : dict
            Additional Data for push notification.
        action : str
            Action for push notification.
        image : str
            Image for push notification. Must be a compound bucket ID to file ID of a jpeg, png, or bmp image in Rilz Storage. It should be formatted as <BUCKET_ID>:<FILE_ID>.
        icon : str
            Icon for push notification. Available only for Android and Web platforms.
        sound : str
            Sound for push notification. Available only for Android and iOS platforms.
        color : str
            Color for push notification. Available only for Android platforms.
        tag : str
            Tag for push notification. Available only for Android platforms.
        badge : float
            Badge for push notification. Available only for iOS platforms.
        draft : bool
            Is message a draft
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        content_available : bool
            If set to true, the notification will be delivered in the background. Available only for iOS Platform.
        critical : bool
            If set to true, the notification will be marked as critical. This requires the app to have the critical notification entitlement. Available only for iOS Platform.
        priority : MessagePriority
            Set the notification priority. "normal" will consider device battery state and may send notifications later. "high" will always attempt to immediately deliver the notification.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/push/{messageId}'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)

        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['title'] = title
        api_params['body'] = body
        api_params['data'] = data
        api_params['action'] = action
        api_params['image'] = image
        api_params['icon'] = icon
        api_params['sound'] = sound
        api_params['color'] = color
        api_params['tag'] = tag
        api_params['badge'] = badge
        api_params['draft'] = draft
        api_params['scheduledAt'] = scheduled_at
        api_params['contentAvailable'] = content_available
        api_params['critical'] = critical
        api_params['priority'] = priority

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_sms(self, message_id: str, content: str, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, draft: bool = None, scheduled_at: str = None) -> Dict[str, Any]:
        """
        Create a new SMS message.

        Parameters
        ----------
        message_id : str
            Message ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        content : str
            SMS Content.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        draft : bool
            Is message a draft
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/sms'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        if content is None:
            raise RilzException('Missing required parameter: "content"')


        api_params['messageId'] = message_id
        api_params['content'] = content
        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['draft'] = draft
        api_params['scheduledAt'] = scheduled_at

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_sms(self, message_id: str, topics: List[str] = None, users: List[str] = None, targets: List[str] = None, content: str = None, draft: bool = None, scheduled_at: str = None) -> Dict[str, Any]:
        """
        Update an SMS message by its unique ID. This endpoint only works on messages that are in draft status. Messages that are already processing, sent, or failed cannot be updated.
        

        Parameters
        ----------
        message_id : str
            Message ID.
        topics : List[str]
            List of Topic IDs.
        users : List[str]
            List of User IDs.
        targets : List[str]
            List of Targets IDs.
        content : str
            Email Content.
        draft : bool
            Is message a draft
        scheduled_at : str
            Scheduled delivery time for message in [ISO 8601](https://www.iso.org/iso-8601-date-and-time-format.html) format. DateTime value must be in future.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/sms/{messageId}'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)

        api_params['topics'] = topics
        api_params['users'] = users
        api_params['targets'] = targets
        api_params['content'] = content
        api_params['draft'] = draft
        api_params['scheduledAt'] = scheduled_at

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_message(self, message_id: str) -> Dict[str, Any]:
        """
        Get a message by its unique ID.
        

        Parameters
        ----------
        message_id : str
            Message ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/{messageId}'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete(self, message_id: str) -> Dict[str, Any]:
        """
        Delete a message. If the message is not a draft or scheduled, but has been sent, this will not recall the message.

        Parameters
        ----------
        message_id : str
            Message ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/{messageId}'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_message_logs(self, message_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the message activity logs listed by its unique ID.

        Parameters
        ----------
        message_id : str
            Message ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/{messageId}/logs'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_targets(self, message_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get a list of the targets associated with a message.

        Parameters
        ----------
        message_id : str
            Message ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: userId, providerId, identifier, providerType
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/messages/{messageId}/targets'
        api_params = {}
        if message_id is None:
            raise RilzException('Missing required parameter: "message_id"')

        api_path = api_path.replace('{messageId}', message_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_providers(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all providers from the current Rilz project.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, provider, type, enabled
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_apns_provider(self, provider_id: str, name: str, auth_key: str = None, auth_key_id: str = None, team_id: str = None, bundle_id: str = None, sandbox: bool = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Apple Push Notification service provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        auth_key : str
            APNS authentication key.
        auth_key_id : str
            APNS authentication key ID.
        team_id : str
            APNS team ID.
        bundle_id : str
            APNS bundle ID.
        sandbox : bool
            Use APNS sandbox environment.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/apns'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['authKey'] = auth_key
        api_params['authKeyId'] = auth_key_id
        api_params['teamId'] = team_id
        api_params['bundleId'] = bundle_id
        api_params['sandbox'] = sandbox
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_apns_provider(self, provider_id: str, name: str = None, enabled: bool = None, auth_key: str = None, auth_key_id: str = None, team_id: str = None, bundle_id: str = None, sandbox: bool = None) -> Dict[str, Any]:
        """
        Update a Apple Push Notification service provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        auth_key : str
            APNS authentication key.
        auth_key_id : str
            APNS authentication key ID.
        team_id : str
            APNS team ID.
        bundle_id : str
            APNS bundle ID.
        sandbox : bool
            Use APNS sandbox environment.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/apns/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['authKey'] = auth_key
        api_params['authKeyId'] = auth_key_id
        api_params['teamId'] = team_id
        api_params['bundleId'] = bundle_id
        api_params['sandbox'] = sandbox

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_fcm_provider(self, provider_id: str, name: str, service_account_json: dict = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Firebase Cloud Messaging provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        service_account_json : dict
            FCM service account JSON.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/fcm'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['serviceAccountJSON'] = service_account_json
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_fcm_provider(self, provider_id: str, name: str = None, enabled: bool = None, service_account_json: dict = None) -> Dict[str, Any]:
        """
        Update a Firebase Cloud Messaging provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        service_account_json : dict
            FCM service account JSON.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/fcm/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['serviceAccountJSON'] = service_account_json

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_mailgun_provider(self, provider_id: str, name: str, api_key: str = None, domain: str = None, is_eu_region: bool = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Mailgun provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        api_key : str
            Mailgun API Key.
        domain : str
            Mailgun Domain.
        is_eu_region : bool
            Set as EU region.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the reply to field for the mail. Default value is sender name. Reply to name must have reply to email as well.
        reply_to_email : str
            Email set in the reply to field for the mail. Default value is sender email. Reply to email must have reply to name as well.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/mailgun'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['apiKey'] = api_key
        api_params['domain'] = domain
        api_params['isEuRegion'] = is_eu_region
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_mailgun_provider(self, provider_id: str, name: str = None, api_key: str = None, domain: str = None, is_eu_region: bool = None, enabled: bool = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None) -> Dict[str, Any]:
        """
        Update a Mailgun provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        api_key : str
            Mailgun API Key.
        domain : str
            Mailgun Domain.
        is_eu_region : bool
            Set as EU region.
        enabled : bool
            Set as enabled.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the reply to field for the mail. Default value is sender name.
        reply_to_email : str
            Email set in the reply to field for the mail. Default value is sender email.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/mailgun/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['apiKey'] = api_key
        api_params['domain'] = domain
        api_params['isEuRegion'] = is_eu_region
        api_params['enabled'] = enabled
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_msg91_provider(self, provider_id: str, name: str, template_id: str = None, sender_id: str = None, auth_key: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new MSG91 provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        template_id : str
            Msg91 template ID
        sender_id : str
            Msg91 sender ID.
        auth_key : str
            Msg91 auth key.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/msg91'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['templateId'] = template_id
        api_params['senderId'] = sender_id
        api_params['authKey'] = auth_key
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_msg91_provider(self, provider_id: str, name: str = None, enabled: bool = None, template_id: str = None, sender_id: str = None, auth_key: str = None) -> Dict[str, Any]:
        """
        Update a MSG91 provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        template_id : str
            Msg91 template ID.
        sender_id : str
            Msg91 sender ID.
        auth_key : str
            Msg91 auth key.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/msg91/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['templateId'] = template_id
        api_params['senderId'] = sender_id
        api_params['authKey'] = auth_key

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_sendgrid_provider(self, provider_id: str, name: str, api_key: str = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Sendgrid provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        api_key : str
            Sendgrid API key.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the reply to field for the mail. Default value is sender name.
        reply_to_email : str
            Email set in the reply to field for the mail. Default value is sender email.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/sendgrid'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['apiKey'] = api_key
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_sendgrid_provider(self, provider_id: str, name: str = None, enabled: bool = None, api_key: str = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None) -> Dict[str, Any]:
        """
        Update a Sendgrid provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        api_key : str
            Sendgrid API key.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the Reply To field for the mail. Default value is Sender Name.
        reply_to_email : str
            Email set in the Reply To field for the mail. Default value is Sender Email.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/sendgrid/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['apiKey'] = api_key
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_smtp_provider(self, provider_id: str, name: str, host: str, port: float = None, username: str = None, password: str = None, encryption: SmtpEncryption = None, auto_tls: bool = None, mailer: str = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new SMTP provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        host : str
            SMTP hosts. Either a single hostname or multiple semicolon-delimited hostnames. You can also specify a different port for each host such as `smtp1.example.com:25;smtp2.example.com`. You can also specify encryption type, for example: `tls://smtp1.example.com:587;ssl://smtp2.example.com:465"`. Hosts will be tried in order.
        port : float
            The default SMTP server port.
        username : str
            Authentication username.
        password : str
            Authentication password.
        encryption : SmtpEncryption
            Encryption type. Can be omitted, 'ssl', or 'tls'
        auto_tls : bool
            Enable SMTP AutoTLS feature.
        mailer : str
            The value to use for the X-Mailer header.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the reply to field for the mail. Default value is sender name.
        reply_to_email : str
            Email set in the reply to field for the mail. Default value is sender email.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/smtp'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        if host is None:
            raise RilzException('Missing required parameter: "host"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['host'] = host
        api_params['port'] = port
        api_params['username'] = username
        api_params['password'] = password
        api_params['encryption'] = encryption
        api_params['autoTLS'] = auto_tls
        api_params['mailer'] = mailer
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_smtp_provider(self, provider_id: str, name: str = None, host: str = None, port: float = None, username: str = None, password: str = None, encryption: SmtpEncryption = None, auto_tls: bool = None, mailer: str = None, from_name: str = None, from_email: str = None, reply_to_name: str = None, reply_to_email: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Update a SMTP provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        host : str
            SMTP hosts. Either a single hostname or multiple semicolon-delimited hostnames. You can also specify a different port for each host such as `smtp1.example.com:25;smtp2.example.com`. You can also specify encryption type, for example: `tls://smtp1.example.com:587;ssl://smtp2.example.com:465"`. Hosts will be tried in order.
        port : float
            SMTP port.
        username : str
            Authentication username.
        password : str
            Authentication password.
        encryption : SmtpEncryption
            Encryption type. Can be 'ssl' or 'tls'
        auto_tls : bool
            Enable SMTP AutoTLS feature.
        mailer : str
            The value to use for the X-Mailer header.
        from_name : str
            Sender Name.
        from_email : str
            Sender email address.
        reply_to_name : str
            Name set in the Reply To field for the mail. Default value is Sender Name.
        reply_to_email : str
            Email set in the Reply To field for the mail. Default value is Sender Email.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/smtp/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['host'] = host
        api_params['port'] = port
        api_params['username'] = username
        api_params['password'] = password
        api_params['encryption'] = encryption
        api_params['autoTLS'] = auto_tls
        api_params['mailer'] = mailer
        api_params['fromName'] = from_name
        api_params['fromEmail'] = from_email
        api_params['replyToName'] = reply_to_name
        api_params['replyToEmail'] = reply_to_email
        api_params['enabled'] = enabled

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_telesign_provider(self, provider_id: str, name: str, xfrom: str = None, customer_id: str = None, api_key: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Telesign provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        xfrom : str
            Sender Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        customer_id : str
            Telesign customer ID.
        api_key : str
            Telesign API key.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/telesign'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['from'] = xfrom
        api_params['customerId'] = customer_id
        api_params['apiKey'] = api_key
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_telesign_provider(self, provider_id: str, name: str = None, enabled: bool = None, customer_id: str = None, api_key: str = None, xfrom: str = None) -> Dict[str, Any]:
        """
        Update a Telesign provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        customer_id : str
            Telesign customer ID.
        api_key : str
            Telesign API key.
        xfrom : str
            Sender number.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/telesign/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['customerId'] = customer_id
        api_params['apiKey'] = api_key
        api_params['from'] = xfrom

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_textmagic_provider(self, provider_id: str, name: str, xfrom: str = None, username: str = None, api_key: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Textmagic provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        xfrom : str
            Sender Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        username : str
            Textmagic username.
        api_key : str
            Textmagic apiKey.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/textmagic'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['from'] = xfrom
        api_params['username'] = username
        api_params['apiKey'] = api_key
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_textmagic_provider(self, provider_id: str, name: str = None, enabled: bool = None, username: str = None, api_key: str = None, xfrom: str = None) -> Dict[str, Any]:
        """
        Update a Textmagic provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        username : str
            Textmagic username.
        api_key : str
            Textmagic apiKey.
        xfrom : str
            Sender number.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/textmagic/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['username'] = username
        api_params['apiKey'] = api_key
        api_params['from'] = xfrom

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_twilio_provider(self, provider_id: str, name: str, xfrom: str = None, account_sid: str = None, auth_token: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Twilio provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        xfrom : str
            Sender Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        account_sid : str
            Twilio account secret ID.
        auth_token : str
            Twilio authentication token.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/twilio'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['from'] = xfrom
        api_params['accountSid'] = account_sid
        api_params['authToken'] = auth_token
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_twilio_provider(self, provider_id: str, name: str = None, enabled: bool = None, account_sid: str = None, auth_token: str = None, xfrom: str = None) -> Dict[str, Any]:
        """
        Update a Twilio provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        account_sid : str
            Twilio account secret ID.
        auth_token : str
            Twilio authentication token.
        xfrom : str
            Sender number.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/twilio/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['accountSid'] = account_sid
        api_params['authToken'] = auth_token
        api_params['from'] = xfrom

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_vonage_provider(self, provider_id: str, name: str, xfrom: str = None, api_key: str = None, api_secret: str = None, enabled: bool = None) -> Dict[str, Any]:
        """
        Create a new Vonage provider.

        Parameters
        ----------
        provider_id : str
            Provider ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Provider name.
        xfrom : str
            Sender Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        api_key : str
            Vonage API key.
        api_secret : str
            Vonage API secret.
        enabled : bool
            Set as enabled.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/vonage'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['providerId'] = provider_id
        api_params['name'] = name
        api_params['from'] = xfrom
        api_params['apiKey'] = api_key
        api_params['apiSecret'] = api_secret
        api_params['enabled'] = enabled

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_vonage_provider(self, provider_id: str, name: str = None, enabled: bool = None, api_key: str = None, api_secret: str = None, xfrom: str = None) -> Dict[str, Any]:
        """
        Update a Vonage provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        name : str
            Provider name.
        enabled : bool
            Set as enabled.
        api_key : str
            Vonage API key.
        api_secret : str
            Vonage API secret.
        xfrom : str
            Sender number.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/vonage/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['name'] = name
        api_params['enabled'] = enabled
        api_params['apiKey'] = api_key
        api_params['apiSecret'] = api_secret
        api_params['from'] = xfrom

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_provider(self, provider_id: str) -> Dict[str, Any]:
        """
        Get a provider by its unique ID.
        

        Parameters
        ----------
        provider_id : str
            Provider ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete_provider(self, provider_id: str) -> Dict[str, Any]:
        """
        Delete a provider by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/{providerId}'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_provider_logs(self, provider_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the provider activity logs listed by its unique ID.

        Parameters
        ----------
        provider_id : str
            Provider ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/providers/{providerId}/logs'
        api_params = {}
        if provider_id is None:
            raise RilzException('Missing required parameter: "provider_id"')

        api_path = api_path.replace('{providerId}', provider_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_subscriber_logs(self, subscriber_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the subscriber activity logs listed by its unique ID.

        Parameters
        ----------
        subscriber_id : str
            Subscriber ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/subscribers/{subscriberId}/logs'
        api_params = {}
        if subscriber_id is None:
            raise RilzException('Missing required parameter: "subscriber_id"')

        api_path = api_path.replace('{subscriberId}', subscriber_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_topics(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all topics from the current Rilz project.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, description, emailTotal, smsTotal, pushTotal
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_topic(self, topic_id: str, name: str, subscribe: List[str] = None) -> Dict[str, Any]:
        """
        Create a new topic.

        Parameters
        ----------
        topic_id : str
            Topic ID. Choose a custom Topic ID or a new Topic ID.
        name : str
            Topic Name.
        subscribe : List[str]
            An array of role strings with subscribe permission. By default all users are granted with any subscribe permission. [learn more about roles](https://rilck.com/docs/permissions#permission-roles). Maximum of 100 roles are allowed, each 64 characters long.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['topicId'] = topic_id
        api_params['name'] = name
        api_params['subscribe'] = subscribe

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_topic(self, topic_id: str) -> Dict[str, Any]:
        """
        Get a topic by its unique ID.
        

        Parameters
        ----------
        topic_id : str
            Topic ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        api_path = api_path.replace('{topicId}', topic_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_topic(self, topic_id: str, name: str = None, subscribe: List[str] = None) -> Dict[str, Any]:
        """
        Update a topic by its unique ID.
        

        Parameters
        ----------
        topic_id : str
            Topic ID.
        name : str
            Topic Name.
        subscribe : List[str]
            An array of role strings with subscribe permission. By default all users are granted with any subscribe permission. [learn more about roles](https://rilck.com/docs/permissions#permission-roles). Maximum of 100 roles are allowed, each 64 characters long.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        api_path = api_path.replace('{topicId}', topic_id)

        api_params['name'] = name
        api_params['subscribe'] = subscribe

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_topic(self, topic_id: str) -> Dict[str, Any]:
        """
        Delete a topic by its unique ID.

        Parameters
        ----------
        topic_id : str
            Topic ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        api_path = api_path.replace('{topicId}', topic_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_topic_logs(self, topic_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the topic activity logs listed by its unique ID.

        Parameters
        ----------
        topic_id : str
            Topic ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}/logs'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        api_path = api_path.replace('{topicId}', topic_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_subscribers(self, topic_id: str, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all subscribers from the current Rilz project.

        Parameters
        ----------
        topic_id : str
            Topic ID. The topic ID subscribed to.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, provider, type, enabled
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}/subscribers'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        api_path = api_path.replace('{topicId}', topic_id)

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_subscriber(self, topic_id: str, subscriber_id: str, target_id: str) -> Dict[str, Any]:
        """
        Create a new subscriber.

        Parameters
        ----------
        topic_id : str
            Topic ID. The topic ID to subscribe to.
        subscriber_id : str
            Subscriber ID. Choose a custom Subscriber ID or a new Subscriber ID.
        target_id : str
            Target ID. The target ID to link to the specified Topic ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}/subscribers'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        if subscriber_id is None:
            raise RilzException('Missing required parameter: "subscriber_id"')

        if target_id is None:
            raise RilzException('Missing required parameter: "target_id"')

        api_path = api_path.replace('{topicId}', topic_id)

        api_params['subscriberId'] = subscriber_id
        api_params['targetId'] = target_id

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_subscriber(self, topic_id: str, subscriber_id: str) -> Dict[str, Any]:
        """
        Get a subscriber by its unique ID.
        

        Parameters
        ----------
        topic_id : str
            Topic ID. The topic ID subscribed to.
        subscriber_id : str
            Subscriber ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}/subscribers/{subscriberId}'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        if subscriber_id is None:
            raise RilzException('Missing required parameter: "subscriber_id"')

        api_path = api_path.replace('{topicId}', topic_id)
        api_path = api_path.replace('{subscriberId}', subscriber_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete_subscriber(self, topic_id: str, subscriber_id: str) -> Dict[str, Any]:
        """
        Delete a subscriber by its unique ID.

        Parameters
        ----------
        topic_id : str
            Topic ID. The topic ID subscribed to.
        subscriber_id : str
            Subscriber ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/messaging/topics/{topicId}/subscribers/{subscriberId}'
        api_params = {}
        if topic_id is None:
            raise RilzException('Missing required parameter: "topic_id"')

        if subscriber_id is None:
            raise RilzException('Missing required parameter: "subscriber_id"')

        api_path = api_path.replace('{topicId}', topic_id)
        api_path = api_path.replace('{subscriberId}', subscriber_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)



from pydantic import BaseModel
from pydantic import PrivateAttr
from .vault import vault
from rilz.logger import logger
from typing import Any, Dict, ClassVar
from pydantic import BaseModel

class Secret(BaseModel):
    """
    Base class for secrets stored in Vault.

    Subclasses should define the expected fields for the secret and
    set the class attribute `vault_path` to the corresponding Vault path.

    Secret values are fetched from Vault automatically when the model is instantiated.
    """
    # Class attributes
    vault_path: ClassVar[str | None] = None  # Define path in subclasses
    _secret_data_cache: ClassVar[Dict[str, Any] | None] = None  # Class-level cache
    
    # Instance-level private attribute to store loaded data
    _loaded_data: Dict[str, Any] = PrivateAttr(default_factory=dict)
    
    def __init__(self, **data):
        # First load secrets from vault
        
        secret_data = self._load_from_vault(type(self).model_fields.keys())        
        # Merge with any explicitly provided values
        merged_data = {**secret_data, **data}
        
        # Initialize the model with merged data        
        super().__init__(**merged_data)
        
    
    def _load_from_vault(self, fields_name:list) -> Dict[str, Any]:
        """Load secret data from vault"""
        cls = self.__class__
        
        # Check if vault_path is set in the class
        if cls.vault_path is None:
            raise ValueError(f"Vault `vault_path` must be set in the class {cls.__name__}.")
        
        # Use cached data if available
        if cls._secret_data_cache is not None:
            return cls._secret_data_cache
        
        # Otherwise load from vault
        try:
            # Ensure vault client is available
            if vault is None:
                logger.warning(f"Vault client is not initialized for {cls.__name__}.")
                return {}
            
            secret_data = vault.get_secret(cls.vault_path, mount_point="secret")
            if secret_data is None:
                logger.warning(f"No secret data found at path '{cls.vault_path}' for {cls.__name__}.")
                cls._secret_data_cache = {}
            else:                
                cls._secret_data_cache = {key: secret_data[key] for key in fields_name if key in secret_data}
                logger.debug(f"Secret for {cls.__name__} fetched and cached successfully.")
            
            return cls._secret_data_cache
        except Exception as e:
            raise ValueError(f"Failed to fetch secret for {cls.__name__} from path '{cls.vault_path}': {e}") from e            
            
    
    @classmethod
    def clear_cache(cls):
        """Clears the cached secret data for this specific class."""
        logger.debug(f"Clearing cache for {cls.__name__}")
        cls._secret_data_cache = None
    
    class Config:
        # Ensure that the model is not allowed to be instantiated with
        # any extra fields not defined in the model
        extra = 'forbid'



class Role:
    """Helper class to generate role strings for `Permission`."""
    @staticmethod
    def any():
        """Grants access to anyone.
        
        This includes authenticated and unauthenticated users.
        """
        return 'any'

    @staticmethod
    def user(id, status = ""):
        """Grants access to a specific user by user ID.

        You can optionally pass verified or unverified for
        `status` to target specific types of users.

        Parameters
        ----------
        id : str
        status : str, optional

        Returns
        -------
        str
        """
        if status:
            return f'user:{id}/{status}'
        return f'user:{id}'

    @staticmethod
    def users(status = ""):
        """Grants access to any authenticated or anonymous user.
        
        You can optionally pass verified or unverified for
        `status` to target specific types of users.

        Parameters
        ----------
        status : str, optional

        Returns
        -------
        str
        """
        if status:
            return f'users/{status}'
        return 'users'
    
    @staticmethod
    def guests():
        """Grants access to any guest user without a session.

        Authenticated users don't have access to this role.

        Returns
        -------
        str
        """
        return 'guests'

    @staticmethod
    def team(id, role = ""):
        """Grants access to a team by team ID.

        You can optionally pass a role for `role` to target
        team members with the specified role.

        Parameters
        ----------
        id : str
        role : str, optional

        Returns
        -------
        str
        """
        if role:
            return f'team:{id}/{role}'
        return f'team:{id}'

    @staticmethod
    def member(id):
        """Grants access to a specific member of a team.

        When the member is removed from the team, they will
        no longer have access.

        Parameters
        ----------
        id : str

        Returns
        -------
        str
        """
        return f'member:{id}'

    @staticmethod
    def label(name):
        """Grants access to a user with the specified label.

        Parameters
        ----------
        name : str

        Returns
        -------
        str
        """
        return f'label:{name}'


from enum import Enum

class CreditCard(Enum):
    AMERICAN_EXPRESS = "amex"
    ARGENCARD = "argencard"
    CABAL = "cabal"
    CENCOSUD = "cencosud"
    DINERS_CLUB = "diners"
    DISCOVER = "discover"
    ELO = "elo"
    HIPERCARD = "hipercard"
    JCB = "jcb"
    MASTERCARD = "mastercard"
    NARANJA = "naranja"
    TARJETA_SHOPPING = "targeta-shopping"
    UNION_CHINA_PAY = "union-china-pay"
    VISA = "visa"
    MIR = "mir"
    MAESTRO = "maestro"
    RUPAY = "rupay"



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException

class Graphql(Service):

    def __init__(self, client) -> None:
        super(Graphql, self).__init__(client)

    def query(self, query: dict) -> Dict[str, Any]:
        """
        Execute a GraphQL mutation.

        Parameters
        ----------
        query : dict
            The query or queries to execute.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/graphql'
        api_params = {}
        if query is None:
            raise RilzException('Missing required parameter: "query"')


        api_params['query'] = query

        return self.client.call('post', api_path, {
            'x-sdk-graphql': 'true',
            'content-type': 'application/json',
        }, api_params)

    def mutation(self, query: dict) -> Dict[str, Any]:
        """
        Execute a GraphQL mutation.

        Parameters
        ----------
        query : dict
            The query or queries to execute.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/graphql/mutation'
        api_params = {}
        if query is None:
            raise RilzException('Missing required parameter: "query"')


        api_params['query'] = query

        return self.client.call('post', api_path, {
            'x-sdk-graphql': 'true',
            'content-type': 'application/json',
        }, api_params)



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException

class Tokens(Service):

    def __init__(self, client) -> None:
        super(Tokens, self).__init__(client)

    def list(self, bucket_id: str, file_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        List all the tokens created for a specific file or bucket. You can use the query params to filter your results.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File unique ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: expire
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/tokens/buckets/{bucketId}/files/{fileId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def create_file_token(self, bucket_id: str, file_id: str, expire: str = None) -> Dict[str, Any]:
        """
        Create a new token. A token is linked to a file. Token can be passed as a header or request get parameter.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File unique ID.
        expire : str
            Token expiry date
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/tokens/buckets/{bucketId}/files/{fileId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['expire'] = expire

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get(self, token_id: str) -> Dict[str, Any]:
        """
        Get a token by its unique ID.

        Parameters
        ----------
        token_id : str
            Token ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/tokens/{tokenId}'
        api_params = {}
        if token_id is None:
            raise RilzException('Missing required parameter: "token_id"')

        api_path = api_path.replace('{tokenId}', token_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update(self, token_id: str, expire: str = None) -> Dict[str, Any]:
        """
        Update a token by its unique ID. Use this endpoint to update a token's expiry date.

        Parameters
        ----------
        token_id : str
            Token unique ID.
        expire : str
            File token expiry date
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/tokens/{tokenId}'
        api_params = {}
        if token_id is None:
            raise RilzException('Missing required parameter: "token_id"')

        api_path = api_path.replace('{tokenId}', token_id)

        api_params['expire'] = expire

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete(self, token_id: str) -> Dict[str, Any]:
        """
        Delete a token by its unique ID.

        Parameters
        ----------
        token_id : str
            Token ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/tokens/{tokenId}'
        api_params = {}
        if token_id is None:
            raise RilzException('Missing required parameter: "token_id"')

        api_path = api_path.replace('{tokenId}', token_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)



import os
import mimetypes

class InputFile:
    @classmethod
    def from_path(cls, path):
        instance = cls()
        instance.path = path
        instance.filename = os.path.basename(path)
        instance.mime_type = mimetypes.guess_type(path)
        instance.source_type = 'path'
        return instance

    @classmethod
    def from_bytes(cls, bytes, filename, mime_type = None):
        instance = cls()
        instance.data = bytes
        instance.filename = filename
        instance.mime_type = mime_type
        instance.source_type = 'bytes'
        return instance


import setuptools

long_description: str

with open("README.md", "r", encoding="utf-8") as readme_file_desc:
    long_description = readme_file_desc.read()
    
setuptools.setup(
  name = 'rilz',
  packages = [
    'rilz',
    'rilz/vault',
    'rilz/services',
    'rilz/encoders',
    'rilz/enums',
    'rilz/browser',
  ],
  version = '11.0.9',
  license='BSD-3-Clause',
  description = 'Rilz is an open-source self-hosted backend server that abstract and simplify complex and repetitive development tasks behind a very simple REST API',
  long_description = long_description,
  long_description_content_type = 'text/markdown',
  author = 'Rilz Team',
  author_email = 'team@rilck.com',
  maintainer = 'Rilz Team',
  maintainer_email = 'team@rilck.com',
  url = 'https://rilck.com/support',
  download_url='https://github.com/rilckztech/rilz-sdk/archive/11.0.9.tar.gz',
  install_requires=[
    'requests',
    'loguru',
    'webdriver-manager',
    'selenium',
    'hvac',
  ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Environment :: Web Environment',
    'Topic :: Software Development',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)



"""Vault exceptions."""

class VaultError(Exception):
    """Base class for Vault exceptions."""

class VaultConnection(VaultError):
    """Exception raised for connection errors."""

class VaultAuthentication(VaultError):
    """Exception raised for authentication errors."""

class InvalidSecretPath(VaultError):
    """Exception raised for invalid secret path errors."""

class Forbidden(VaultError):
    """Exception raised for forbidden errors."""


from enum import Enum

class ImageGravity(Enum):
    CENTER = "center"
    TOP_LEFT = "top-left"
    TOP = "top"
    TOP_RIGHT = "top-right"
    LEFT = "left"
    RIGHT = "right"
    BOTTOM_LEFT = "bottom-left"
    BOTTOM = "bottom"
    BOTTOM_RIGHT = "bottom-right"



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.compression import Compression;
from ..input_file import InputFile
from ..enums.image_gravity import ImageGravity;
from ..enums.image_format import ImageFormat;

class Storage(Service):

    def __init__(self, client) -> None:
        super(Storage, self).__init__(client)

    def list_buckets(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the storage buckets. You can use the query params to filter your results.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: enabled, name, fileSecurity, maximumFileSize, encryption, antivirus
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_bucket(self, bucket_id: str, name: str, permissions: List[str] = None, file_security: bool = None, enabled: bool = None, maximum_file_size: float = None, allowed_file_extensions: List[str] = None, compression: Compression = None, encryption: bool = None, antivirus: bool = None) -> Dict[str, Any]:
        """
        Create a new storage bucket.

        Parameters
        ----------
        bucket_id : str
            Unique Id. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Bucket name
        permissions : List[str]
            An array of permission strings. By default, no user is granted with any permissions. [Learn more about permissions](https://rilck.com/docs/permissions).
        file_security : bool
            Enables configuring permissions for individual file. A user needs one of file or bucket level permissions to access a file. [Learn more about permissions](https://rilck.com/docs/permissions).
        enabled : bool
            Is bucket enabled? When set to 'disabled', users cannot access the files in this bucket but Server SDKs with and API key can still access the bucket. No files are lost when this is toggled.
        maximum_file_size : float
            Maximum file size allowed in bytes. Maximum allowed value is 30MB.
        allowed_file_extensions : List[str]
            Allowed file extensions. Maximum of 100 extensions are allowed, each 64 characters long.
        compression : Compression
            Compression algorithm choosen for compression. Can be one of none,  [gzip](https://en.wikipedia.org/wiki/Gzip), or [zstd](https://en.wikipedia.org/wiki/Zstd), For file size above 20MB compression is skipped even if it's enabled
        encryption : bool
            Is encryption enabled? For file size above 20MB encryption is skipped even if it's enabled
        antivirus : bool
            Is virus scanning enabled? For file size above 20MB AntiVirus scanning is skipped even if it's enabled
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['bucketId'] = bucket_id
        api_params['name'] = name
        api_params['permissions'] = permissions
        api_params['fileSecurity'] = file_security
        api_params['enabled'] = enabled
        api_params['maximumFileSize'] = maximum_file_size
        api_params['allowedFileExtensions'] = allowed_file_extensions
        api_params['compression'] = compression
        api_params['encryption'] = encryption
        api_params['antivirus'] = antivirus

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_bucket(self, bucket_id: str) -> Dict[str, Any]:
        """
        Get a storage bucket by its unique ID. This endpoint response returns a JSON object with the storage bucket metadata.

        Parameters
        ----------
        bucket_id : str
            Bucket unique ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_bucket(self, bucket_id: str, name: str, permissions: List[str] = None, file_security: bool = None, enabled: bool = None, maximum_file_size: float = None, allowed_file_extensions: List[str] = None, compression: Compression = None, encryption: bool = None, antivirus: bool = None) -> Dict[str, Any]:
        """
        Update a storage bucket by its unique ID.

        Parameters
        ----------
        bucket_id : str
            Bucket unique ID.
        name : str
            Bucket name
        permissions : List[str]
            An array of permission strings. By default, the current permissions are inherited. [Learn more about permissions](https://rilck.com/docs/permissions).
        file_security : bool
            Enables configuring permissions for individual file. A user needs one of file or bucket level permissions to access a file. [Learn more about permissions](https://rilck.com/docs/permissions).
        enabled : bool
            Is bucket enabled? When set to 'disabled', users cannot access the files in this bucket but Server SDKs with and API key can still access the bucket. No files are lost when this is toggled.
        maximum_file_size : float
            Maximum file size allowed in bytes. Maximum allowed value is 30MB.
        allowed_file_extensions : List[str]
            Allowed file extensions. Maximum of 100 extensions are allowed, each 64 characters long.
        compression : Compression
            Compression algorithm choosen for compression. Can be one of none, [gzip](https://en.wikipedia.org/wiki/Gzip), or [zstd](https://en.wikipedia.org/wiki/Zstd), For file size above 20MB compression is skipped even if it's enabled
        encryption : bool
            Is encryption enabled? For file size above 20MB encryption is skipped even if it's enabled
        antivirus : bool
            Is virus scanning enabled? For file size above 20MB AntiVirus scanning is skipped even if it's enabled
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        api_path = api_path.replace('{bucketId}', bucket_id)

        api_params['name'] = name
        api_params['permissions'] = permissions
        api_params['fileSecurity'] = file_security
        api_params['enabled'] = enabled
        api_params['maximumFileSize'] = maximum_file_size
        api_params['allowedFileExtensions'] = allowed_file_extensions
        api_params['compression'] = compression
        api_params['encryption'] = encryption
        api_params['antivirus'] = antivirus

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_bucket(self, bucket_id: str) -> Dict[str, Any]:
        """
        Delete a storage bucket by its unique ID.

        Parameters
        ----------
        bucket_id : str
            Bucket unique ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_files(self, bucket_id: str, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the user files. You can use the query params to filter your results.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, signature, mimeType, sizeOriginal, chunksTotal, chunksUploaded
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_file(self, bucket_id: str, file_id: str, file: InputFile, permissions: List[str] = None, on_progress = None) -> Dict[str, Any]:
        """
        Create a new file. Before using this route, you should create a new bucket resource using either a [server integration](https://rilck.com/docs/server/storage#storageCreateBucket) API or directly from your Rilz console.
        
        Larger files should be uploaded using multiple requests with the [content-range](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Range) header to send a partial request with a maximum supported chunk of `5MB`. The `content-range` header values should always be in bytes.
        
        When the first request is sent, the server will return the **File** object, and the subsequent part request must include the file's **id** in `x-rilz-id` header to allow the server to know that the partial upload is for the existing file and not for a new one.
        
        If you're creating a new file using one of the Rilz SDKs, all the chunking logic will be managed by the SDK internally.
        

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        file : InputFile
            Binary file. Rilz SDKs provide helpers to handle file input. [Learn about file input](https://rilck.com/docs/products/storage/upload-download#input-file).
        permissions : List[str]
            An array of permission strings. By default, only the current user is granted all permissions. [Learn more about permissions](https://rilck.com/docs/permissions).
                on_progress : callable, optional
            Optional callback for upload progress
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        if file is None:
            raise RilzException('Missing required parameter: "file"')

        api_path = api_path.replace('{bucketId}', bucket_id)

        api_params['fileId'] = file_id
        api_params['file'] = str(file).lower() if type(file) is bool else file
        api_params['permissions'] = permissions

        param_name = 'file'


        upload_id = ''
        upload_id = file_id

        return self.client.chunked_upload(api_path, {
            'content-type': 'multipart/form-data',
        }, api_params, param_name, on_progress, upload_id)

    def get_file(self, bucket_id: str, file_id: str) -> Dict[str, Any]:
        """
        Get a file by its unique ID. This endpoint response returns a JSON object with the file metadata.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_file(self, bucket_id: str, file_id: str, name: str = None, permissions: List[str] = None) -> Dict[str, Any]:
        """
        Update a file by its unique ID. Only users with write permissions have access to update this resource.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File unique ID.
        name : str
            Name of the file
        permissions : List[str]
            An array of permission string. By default, the current permissions are inherited. [Learn more about permissions](https://rilck.com/docs/permissions).
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['name'] = name
        api_params['permissions'] = permissions

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_file(self, bucket_id: str, file_id: str) -> Dict[str, Any]:
        """
        Delete a file by its unique ID. Only users with write permissions have access to delete this resource.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_file_download(self, bucket_id: str, file_id: str, token: str = None) -> bytes:
        """
        Get a file content by its unique ID. The endpoint response return with a 'Content-Disposition: attachment' header that tells the browser to start downloading the file to user downloads directory.

        Parameters
        ----------
        bucket_id : str
            Storage bucket ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID.
        token : str
            File token for accessing this file.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}/download'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['token'] = token

        return self.client.call('get', api_path, {
        }, api_params)

    def get_file_preview(self, bucket_id: str, file_id: str, width: float = None, height: float = None, gravity: ImageGravity = None, quality: float = None, border_width: float = None, border_color: str = None, border_radius: float = None, opacity: float = None, rotation: float = None, background: str = None, output: ImageFormat = None, token: str = None) -> bytes:
        """
        Get a file preview image. Currently, this method supports preview for image files (jpg, png, and gif), other supported formats, like pdf, docs, slides, and spreadsheets, will return the file icon image. You can also pass query string arguments for cutting and resizing your preview image. Preview is supported only for image files smaller than 10MB.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID
        width : float
            Resize preview image width, Pass an integer between 0 to 4000.
        height : float
            Resize preview image height, Pass an integer between 0 to 4000.
        gravity : ImageGravity
            Image crop gravity. Can be one of center,top-left,top,top-right,left,right,bottom-left,bottom,bottom-right
        quality : float
            Preview image quality. Pass an integer between 0 to 100. Defaults to keep existing image quality.
        border_width : float
            Preview image border in pixels. Pass an integer between 0 to 100. Defaults to 0.
        border_color : str
            Preview image border color. Use a valid HEX color, no # is needed for prefix.
        border_radius : float
            Preview image border radius in pixels. Pass an integer between 0 to 4000.
        opacity : float
            Preview image opacity. Only works with images having an alpha channel (like png). Pass a number between 0 to 1.
        rotation : float
            Preview image rotation in degrees. Pass an integer between -360 and 360.
        background : str
            Preview image background color. Only works with transparent images (png). Use a valid HEX color, no # is needed for prefix.
        output : ImageFormat
            Output format type (jpeg, jpg, png, gif and webp).
        token : str
            File token for accessing this file.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}/preview'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['width'] = width
        api_params['height'] = height
        api_params['gravity'] = gravity
        api_params['quality'] = quality
        api_params['borderWidth'] = border_width
        api_params['borderColor'] = border_color
        api_params['borderRadius'] = border_radius
        api_params['opacity'] = opacity
        api_params['rotation'] = rotation
        api_params['background'] = background
        api_params['output'] = output
        api_params['token'] = token

        return self.client.call('get', api_path, {
        }, api_params)

    def get_file_view(self, bucket_id: str, file_id: str, token: str = None) -> bytes:
        """
        Get a file content by its unique ID. This endpoint is similar to the download method but returns with no  'Content-Disposition: attachment' header.

        Parameters
        ----------
        bucket_id : str
            Storage bucket unique ID. You can create a new storage bucket using the Storage service [server integration](https://rilck.com/docs/server/storage#createBucket).
        file_id : str
            File ID.
        token : str
            File token for accessing this file.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/storage/buckets/{bucketId}/files/{fileId}/view'
        api_params = {}
        if bucket_id is None:
            raise RilzException('Missing required parameter: "bucket_id"')

        if file_id is None:
            raise RilzException('Missing required parameter: "file_id"')

        api_path = api_path.replace('{bucketId}', bucket_id)
        api_path = api_path.replace('{fileId}', file_id)

        api_params['token'] = token

        return self.client.call('get', api_path, {
        }, api_params)



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException

class Teams(Service):

    def __init__(self, client) -> None:
        super(Teams, self).__init__(client)

    def list(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the teams in which the current user is a member. You can use the parameters to filter your results.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, total, billingPlan
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create(self, team_id: str, name: str, roles: List[str] = None) -> Dict[str, Any]:
        """
        Create a new team. The user who creates the team will automatically be assigned as the owner of the team. Only the users with the owner role can invite new members, add new owners and delete or update the team.

        Parameters
        ----------
        team_id : str
            Team ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Team name. Max length: 128 chars.
        roles : List[str]
            Array of strings. Use this param to set the roles in the team for the user who created it. The default role is **owner**. A role can be any string. Learn more about [roles and permissions](https://rilck.com/docs/permissions). Maximum of 100 roles are allowed, each 32 characters long.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['teamId'] = team_id
        api_params['name'] = name
        api_params['roles'] = roles

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get(self, team_id: str) -> Dict[str, Any]:
        """
        Get a team by its ID. All team members have read access for this resource.

        Parameters
        ----------
        team_id : str
            Team ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        api_path = api_path.replace('{teamId}', team_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_name(self, team_id: str, name: str) -> Dict[str, Any]:
        """
        Update the team's name by its unique ID.

        Parameters
        ----------
        team_id : str
            Team ID.
        name : str
            New team name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        api_path = api_path.replace('{teamId}', team_id)

        api_params['name'] = name

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete(self, team_id: str) -> Dict[str, Any]:
        """
        Delete a team using its ID. Only team members with the owner role can delete the team.

        Parameters
        ----------
        team_id : str
            Team ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        api_path = api_path.replace('{teamId}', team_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_memberships(self, team_id: str, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Use this endpoint to list a team's members using the team's ID. All team members have read access to this endpoint. Hide sensitive attributes from the response by toggling membership privacy in the Console.

        Parameters
        ----------
        team_id : str
            Team ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: userId, teamId, invited, joined, confirm, roles
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        api_path = api_path.replace('{teamId}', team_id)

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_membership(self, team_id: str, roles: List[str], email: str = None, user_id: str = None, phone: str = None, url: str = None, name: str = None) -> Dict[str, Any]:
        """
        Invite a new member to join your team. Provide an ID for existing users, or invite unregistered users using an email or phone number. If initiated from a Client SDK, Rilz will send an email or sms with a link to join the team to the invited user, and an account will be created for them if one doesn't exist. If initiated from a Server SDK, the new member will be added automatically to the team.
        
        You only need to provide one of a user ID, email, or phone number. Rilz will prioritize accepting the user ID > email > phone number if you provide more than one of these parameters.
        
        Use the `url` parameter to redirect the user from the invitation email to your app. After the user is redirected, use the [Update Team Membership Status](https://rilck.com/docs/references/cloud/client-web/teams#updateMembershipStatus) endpoint to allow the user to accept the invitation to the team. 
        
        Please note that to avoid a [Redirect Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md) Rilz will accept the only redirect URLs under the domains you have added as a platform on the Rilz Console.
        

        Parameters
        ----------
        team_id : str
            Team ID.
        roles : List[str]
            Array of strings. Use this param to set the user roles in the team. A role can be any string. Learn more about [roles and permissions](https://rilck.com/docs/permissions). Maximum of 100 roles are allowed, each 32 characters long.
        email : str
            Email of the new team member.
        user_id : str
            ID of the user to be added to a team.
        phone : str
            Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        url : str
            URL to redirect the user back to your app from the invitation email. This parameter is not required when an API key is supplied. Only URLs from hostnames in your project platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        name : str
            Name of the new team member. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if roles is None:
            raise RilzException('Missing required parameter: "roles"')

        api_path = api_path.replace('{teamId}', team_id)

        api_params['email'] = email
        api_params['userId'] = user_id
        api_params['phone'] = phone
        api_params['roles'] = roles
        api_params['url'] = url
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_membership(self, team_id: str, membership_id: str) -> Dict[str, Any]:
        """
        Get a team member by the membership unique id. All team members have read access for this resource. Hide sensitive attributes from the response by toggling membership privacy in the Console.

        Parameters
        ----------
        team_id : str
            Team ID.
        membership_id : str
            Membership ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships/{membershipId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if membership_id is None:
            raise RilzException('Missing required parameter: "membership_id"')

        api_path = api_path.replace('{teamId}', team_id)
        api_path = api_path.replace('{membershipId}', membership_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_membership(self, team_id: str, membership_id: str, roles: List[str]) -> Dict[str, Any]:
        """
        Modify the roles of a team member. Only team members with the owner role have access to this endpoint. Learn more about [roles and permissions](https://rilck.com/docs/permissions).
        

        Parameters
        ----------
        team_id : str
            Team ID.
        membership_id : str
            Membership ID.
        roles : List[str]
            An array of strings. Use this param to set the user's roles in the team. A role can be any string. Learn more about [roles and permissions](https://rilck.com/docs/permissions). Maximum of 100 roles are allowed, each 32 characters long.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships/{membershipId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if membership_id is None:
            raise RilzException('Missing required parameter: "membership_id"')

        if roles is None:
            raise RilzException('Missing required parameter: "roles"')

        api_path = api_path.replace('{teamId}', team_id)
        api_path = api_path.replace('{membershipId}', membership_id)

        api_params['roles'] = roles

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_membership(self, team_id: str, membership_id: str) -> Dict[str, Any]:
        """
        This endpoint allows a user to leave a team or for a team owner to delete the membership of any other team member. You can also use this endpoint to delete a user membership even if it is not accepted.

        Parameters
        ----------
        team_id : str
            Team ID.
        membership_id : str
            Membership ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships/{membershipId}'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if membership_id is None:
            raise RilzException('Missing required parameter: "membership_id"')

        api_path = api_path.replace('{teamId}', team_id)
        api_path = api_path.replace('{membershipId}', membership_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_membership_status(self, team_id: str, membership_id: str, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to allow a user to accept an invitation to join a team after being redirected back to your app from the invitation email received by the user.
        
        If the request is successful, a session for the user is automatically created.
        

        Parameters
        ----------
        team_id : str
            Team ID.
        membership_id : str
            Membership ID.
        user_id : str
            User ID.
        secret : str
            Secret key.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/memberships/{membershipId}/status'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if membership_id is None:
            raise RilzException('Missing required parameter: "membership_id"')

        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')

        api_path = api_path.replace('{teamId}', team_id)
        api_path = api_path.replace('{membershipId}', membership_id)

        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_prefs(self, team_id: str) -> Dict[str, Any]:
        """
        Get the team's shared preferences by its unique ID. If a preference doesn't need to be shared by all team members, prefer storing them in [user preferences](https://rilck.com/docs/references/cloud/client-web/account#getPrefs).

        Parameters
        ----------
        team_id : str
            Team ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/prefs'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        api_path = api_path.replace('{teamId}', team_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_prefs(self, team_id: str, prefs: dict) -> Dict[str, Any]:
        """
        Update the team's preferences by its unique ID. The object you pass is stored as is and replaces any previous value. The maximum allowed prefs size is 64kB and throws an error if exceeded.

        Parameters
        ----------
        team_id : str
            Team ID.
        prefs : dict
            Prefs key-value JSON object.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/teams/{teamId}/prefs'
        api_params = {}
        if team_id is None:
            raise RilzException('Missing required parameter: "team_id"')

        if prefs is None:
            raise RilzException('Missing required parameter: "prefs"')

        api_path = api_path.replace('{teamId}', team_id)

        api_params['prefs'] = prefs

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)



from enum import Enum

class Flag(Enum):
    AFGHANISTAN = "af"
    ANGOLA = "ao"
    ALBANIA = "al"
    ANDORRA = "ad"
    UNITED_ARAB_EMIRATES = "ae"
    ARGENTINA = "ar"
    ARMENIA = "am"
    ANTIGUA_AND_BARBUDA = "ag"
    AUSTRALIA = "au"
    AUSTRIA = "at"
    AZERBAIJAN = "az"
    BURUNDI = "bi"
    BELGIUM = "be"
    BENIN = "bj"
    BURKINA_FASO = "bf"
    BANGLADESH = "bd"
    BULGARIA = "bg"
    BAHRAIN = "bh"
    BAHAMAS = "bs"
    BOSNIA_AND_HERZEGOVINA = "ba"
    BELARUS = "by"
    BELIZE = "bz"
    BOLIVIA = "bo"
    BRAZIL = "br"
    BARBADOS = "bb"
    BRUNEI_DARUSSALAM = "bn"
    BHUTAN = "bt"
    BOTSWANA = "bw"
    CENTRAL_AFRICAN_REPUBLIC = "cf"
    CANADA = "ca"
    SWITZERLAND = "ch"
    CHILE = "cl"
    CHINA = "cn"
    COTE_DIVOIRE = "ci"
    CAMEROON = "cm"
    DEMOCRATIC_REPUBLIC_OF_THE_CONGO = "cd"
    REPUBLIC_OF_THE_CONGO = "cg"
    COLOMBIA = "co"
    COMOROS = "km"
    CAPE_VERDE = "cv"
    COSTA_RICA = "cr"
    CUBA = "cu"
    CYPRUS = "cy"
    CZECH_REPUBLIC = "cz"
    GERMANY = "de"
    DJIBOUTI = "dj"
    DOMINICA = "dm"
    DENMARK = "dk"
    DOMINICAN_REPUBLIC = "do"
    ALGERIA = "dz"
    ECUADOR = "ec"
    EGYPT = "eg"
    ERITREA = "er"
    SPAIN = "es"
    ESTONIA = "ee"
    ETHIOPIA = "et"
    FINLAND = "fi"
    FIJI = "fj"
    FRANCE = "fr"
    MICRONESIA_FEDERATED_STATES_OF = "fm"
    GABON = "ga"
    UNITED_KINGDOM = "gb"
    GEORGIA = "ge"
    GHANA = "gh"
    GUINEA = "gn"
    GAMBIA = "gm"
    GUINEA_BISSAU = "gw"
    EQUATORIAL_GUINEA = "gq"
    GREECE = "gr"
    GRENADA = "gd"
    GUATEMALA = "gt"
    GUYANA = "gy"
    HONDURAS = "hn"
    CROATIA = "hr"
    HAITI = "ht"
    HUNGARY = "hu"
    INDONESIA = "id"
    INDIA = "in"
    IRELAND = "ie"
    IRAN_ISLAMIC_REPUBLIC_OF = "ir"
    IRAQ = "iq"
    ICELAND = "is"
    ISRAEL = "il"
    ITALY = "it"
    JAMAICA = "jm"
    JORDAN = "jo"
    JAPAN = "jp"
    KAZAKHSTAN = "kz"
    KENYA = "ke"
    KYRGYZSTAN = "kg"
    CAMBODIA = "kh"
    KIRIBATI = "ki"
    SAINT_KITTS_AND_NEVIS = "kn"
    SOUTH_KOREA = "kr"
    KUWAIT = "kw"
    LAO_PEOPLES_DEMOCRATIC_REPUBLIC = "la"
    LEBANON = "lb"
    LIBERIA = "lr"
    LIBYA = "ly"
    SAINT_LUCIA = "lc"
    LIECHTENSTEIN = "li"
    SRI_LANKA = "lk"
    LESOTHO = "ls"
    LITHUANIA = "lt"
    LUXEMBOURG = "lu"
    LATVIA = "lv"
    MOROCCO = "ma"
    MONACO = "mc"
    MOLDOVA = "md"
    MADAGASCAR = "mg"
    MALDIVES = "mv"
    MEXICO = "mx"
    MARSHALL_ISLANDS = "mh"
    NORTH_MACEDONIA = "mk"
    MALI = "ml"
    MALTA = "mt"
    MYANMAR = "mm"
    MONTENEGRO = "me"
    MONGOLIA = "mn"
    MOZAMBIQUE = "mz"
    MAURITANIA = "mr"
    MAURITIUS = "mu"
    MALAWI = "mw"
    MALAYSIA = "my"
    NAMIBIA = "na"
    NIGER = "ne"
    NIGERIA = "ng"
    NICARAGUA = "ni"
    NETHERLANDS = "nl"
    NORWAY = "no"
    NEPAL = "np"
    NAURU = "nr"
    NEW_ZEALAND = "nz"
    OMAN = "om"
    PAKISTAN = "pk"
    PANAMA = "pa"
    PERU = "pe"
    PHILIPPINES = "ph"
    PALAU = "pw"
    PAPUA_NEW_GUINEA = "pg"
    POLAND = "pl"
    FRENCH_POLYNESIA = "pf"
    NORTH_KOREA = "kp"
    PORTUGAL = "pt"
    PARAGUAY = "py"
    QATAR = "qa"
    ROMANIA = "ro"
    RUSSIA = "ru"
    RWANDA = "rw"
    SAUDI_ARABIA = "sa"
    SUDAN = "sd"
    SENEGAL = "sn"
    SINGAPORE = "sg"
    SOLOMON_ISLANDS = "sb"
    SIERRA_LEONE = "sl"
    EL_SALVADOR = "sv"
    SAN_MARINO = "sm"
    SOMALIA = "so"
    SERBIA = "rs"
    SOUTH_SUDAN = "ss"
    SAO_TOME_AND_PRINCIPE = "st"
    SURINAME = "sr"
    SLOVAKIA = "sk"
    SLOVENIA = "si"
    SWEDEN = "se"
    ESWATINI = "sz"
    SEYCHELLES = "sc"
    SYRIA = "sy"
    CHAD = "td"
    TOGO = "tg"
    THAILAND = "th"
    TAJIKISTAN = "tj"
    TURKMENISTAN = "tm"
    TIMOR_LESTE = "tl"
    TONGA = "to"
    TRINIDAD_AND_TOBAGO = "tt"
    TUNISIA = "tn"
    TURKEY = "tr"
    TUVALU = "tv"
    TANZANIA = "tz"
    UGANDA = "ug"
    UKRAINE = "ua"
    URUGUAY = "uy"
    UNITED_STATES = "us"
    UZBEKISTAN = "uz"
    VATICAN_CITY = "va"
    SAINT_VINCENT_AND_THE_GRENADINES = "vc"
    VENEZUELA = "ve"
    VIETNAM = "vn"
    VANUATU = "vu"
    SAMOA = "ws"
    YEMEN = "ye"
    SOUTH_AFRICA = "za"
    ZAMBIA = "zm"
    ZIMBABWE = "zw"



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException

class Locale(Service):

    def __init__(self, client) -> None:
        super(Locale, self).__init__(client)

    def get(self) -> Dict[str, Any]:
        """
        Get the current user location based on IP. Returns an object with user country code, country name, continent name, continent code, ip address and suggested currency. You can use the locale header to get the data in a supported language.
        
        ([IP Geolocation by DB-IP](https://db-ip.com))

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_codes(self) -> Dict[str, Any]:
        """
        List of all locale codes in [ISO 639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes).

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/codes'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_continents(self) -> Dict[str, Any]:
        """
        List of all continents. You can use the locale header to get the data in a supported language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/continents'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_countries(self) -> Dict[str, Any]:
        """
        List of all countries. You can use the locale header to get the data in a supported language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/countries'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_countries_eu(self) -> Dict[str, Any]:
        """
        List of all countries that are currently members of the EU. You can use the locale header to get the data in a supported language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/countries/eu'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_countries_phones(self) -> Dict[str, Any]:
        """
        List of all countries phone codes. You can use the locale header to get the data in a supported language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/countries/phones'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_currencies(self) -> Dict[str, Any]:
        """
        List of all currencies, including currency symbol, name, plural, and decimal digits for all major and minor currencies. You can use the locale header to get the data in a supported language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/currencies'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_languages(self) -> Dict[str, Any]:
        """
        List of all languages classified by ISO 639-1 including 2-letter code, name in English, and name in the respective language.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/locale/languages'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)
